<?php declare(strict_types=1);

namespace Everyware\Installer\Console;

use ZipArchive;
use RuntimeException;
use GuzzleHttp\Client;
use Symfony\Component\Process\Process;
use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\Exception\IOExceptionInterface;

class CreateCommand extends Command
{
    private const NAME_PREFIX = 'everyware-customer-project';

    protected static $descriptions = [
        'name' => 'The customer name',
        'version' => 'The version of Starter Kit to install from : "master" is available to fetch latest commit.',
        'deployment' => 'The name of the deployment wanted common values : stage, test, prod.',
        'force' => 'Forces install even if the directory already exists',
        'path' => 'The path to where to create the project.',
        'prefix' => 'The name will be prefixed with "everyware-customer-project" unless specified.'
    ];

    private $rootDir;
    private $projectName;
    private $version;


    /**
     * Configure the command options.
     *
     * @return void
     */
    protected function configure(): void
    {
        $this
            ->setName('create')
            ->setDescription('Create a new Everyware project')
            ->addArgument('name', InputArgument::REQUIRED, static::$descriptions['name'])
            ->addOption('from-version', null, InputOption::VALUE_OPTIONAL, static::$descriptions['version'], 'latest')
            ->addOption('deployment', null, InputOption::VALUE_OPTIONAL, static::$descriptions['deployment'], 'test')
            ->addOption('force', 'f', InputOption::VALUE_NONE, static::$descriptions['force'])
            ->addOption('path', null, InputOption::VALUE_OPTIONAL, static::$descriptions['path'], '~/')
            ->addOption('prefix', null, InputOption::VALUE_OPTIONAL, static::$descriptions['prefix'],
                static::NAME_PREFIX);
    }

    /**
     * Execute the command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     *
     * @return void
     */
    protected function execute(InputInterface $input, OutputInterface $output): void
    {
        $this->setupValues($input);

        $installationDir = $this->getInstallationDir();

        if ( ! $input->getOption('force')) {
            $this->verifyApplicationDoesntExist($installationDir);
        }

        $cmdOptions = implode(' ', [
            '--customer ' . $this->projectName,
            '--deployment ' . $input->getOption('deployment'),
            '--version ' . $this->version,
            '--path ' . $this->rootDir,
        ]);

        $commands = [
            "./ctl.sh {$cmdOptions} create",
        ];

        if ($input->getOption('no-ansi')) {
            $commands = array_map(function ($value) {
                return $value . ' --no-ansi';
            }, $commands);
        }

        if ($input->getOption('quiet')) {
            $commands = array_map(function ($value) {
                return $value . ' --quiet';
            }, $commands);
        }

        $process = Process::fromShellCommandline(implode(' && ', $commands), null, null, null, null);

        $output->writeln('<comment>Install Project to: ' . $installationDir . ' from Starter kit: ' . $this->version . '</comment>');
        $process->run(function ($type, $line) use ($output) {
            $output->write($line);
        });

        $output->writeln('<comment>Project ready for you at ' . $installationDir . '</comment>');
    }

    /**
     * Generate the project name.
     *
     * @param InputInterface $input
     *
     * @return string
     */
    protected function generateProjectName(InputInterface $input): string
    {
        $projectName = $input->getArgument('name');
        $prefix = $input->getOption('prefix');

        return "{$prefix}-{$projectName}";
    }

    /**
     * Verify that the application does not already exist.
     *
     * @param  string $directory
     *
     * @return void
     */
    protected function verifyApplicationDoesntExist($directory): void
    {
        if (getcwd() !== $directory && (is_dir($directory) || is_file($directory))) {
            throw new RuntimeException('Project already exists!');
        }
    }

    /**
     * Get the composer command for the environment.
     *
     * @return string
     */
    protected function findComposer(): string
    {
        if (file_exists(getcwd() . '/composer.phar')) {
            return '"' . PHP_BINARY . '" composer.phar';
        }

        return 'composer';
    }

    private function getInstallationDir(): string
    {
        return "{$this->rootDir}/{$this->projectName}";
    }

    protected function setupValues(InputInterface $input)
    {
        $this->rootDir = $this->generateRootDir($input);
        $this->projectName = $this->generateProjectName($input);
        $this->version = $input->getOption('from-version');

        return $this;
    }

    private function generateRootDir(InputInterface $input)
    {
        return rtrim(str_replace('~', $_SERVER['HOME'], $input->getOption('path')), '/');
    }
}
